@@ -21,18 +21,23 @@
 #include <stdarg.h>
 #include <math.h>
 
+#define NEEDS_BABL_DB
 #include "babl-internal.h"
 #include "babl-db.h"
 
-static int
-each_babl_format_destroy (Babl *babl,
-                          void *data)
+static int babl_format_destruct (void *babl)
 {
-  if (babl->format.from_list)
-    babl_list_destroy (babl->format.from_list);
-  babl_free (babl);
-
-  return 0;  /* continue iterating */
+  BablFormat *format = babl;
+  if (format->image_template != NULL)
+    {
+      babl_set_destructor (format->image_template, NULL);
+      /* with no destructor set, the circular reference is no problem */
+      babl_free (format->image_template);
+      format->image_template = NULL;
+    }
+  if (format->from_list)
+    babl_free (format->from_list);
+  return 0;
 }
 
 static Babl *
@@ -43,11 +48,11 @@ format_new (const char     *name,
             BablModel      *model,
             BablComponent **component,
             BablSampling  **sampling,
-            BablType      **type)
+            const BablType **type)
 {
   Babl *babl;
 
-  /* i is desintation position */
+  /* i is destination position */
   int i, j, component_found = 0;
   for (i = 0; i < model->components; i++)
     {
@@ -62,8 +67,10 @@ format_new (const char     *name,
       if (!component_found)
         {
           component_found = 0;
-          babl_fatal ("matching source component for %s in model %s not found",
+#if 0 /* since babl does the right thing, let it happen */
+          babl_fatal("matching source component for %s in model %s not found",
                       model->component[i]->instance.name, model->instance.name);
+#endif
         }
     }
 
@@ -75,6 +82,7 @@ format_new (const char     *name,
                       sizeof (BablType *) * (components) +
                       sizeof (int) * (components) +
                       sizeof (int) * (components));
+  babl_set_destructor (babl, babl_format_destruct);
 
   babl->format.from_list = NULL;
   babl->format.component = (void *) (((char *) babl) + sizeof (BablFormat));
@@ -106,24 +114,24 @@ format_new (const char     *name,
   babl->format.loss = -1.0;
   babl->format.visited = 0;
   babl->format.image_template = NULL;
+  babl->format.format_n = 0;
 
   return babl;
 }
 
 
-
 static char *
-create_name (BablModel      *model,
-             int             components,
-             BablComponent **component,
-             BablType      **type)
+create_name (const BablModel *model,
+             int              components,
+             BablComponent  **component,
+             const BablType **type)
 {
   char            buf[512] = "";
   char           *p = &buf[0];
   int             i;
   int             same_types = 1;
-  BablType      **t          = type;
-  BablType       *first_type = *type;
+  const BablType**t          = type;
+  const BablType *first_type = *type;
   BablComponent **c1         = component;
   BablComponent **c2         = model->component;
 
@@ -180,8 +188,109 @@ create_name (BablModel      *model,
   return babl_strdup (buf);
 }
 
-Babl *
-babl_format_new (void *first_arg,
+
+static char *
+ncomponents_create_name (const Babl *type,
+                         int         components)
+{
+  char buf[512];
+  sprintf (buf, "%s[%i] ", type->instance.name, components);
+  return babl_strdup (buf);
+}
+
+static void
+babl_format_set_is_format_n (Babl *format)
+{
+  if (format->class_type == BABL_FORMAT)
+    {
+      format->format.format_n = 1;
+    }
+}
+
+const Babl *
+babl_format_n (const Babl *btype,
+               int   components)
+{
+  int            i;
+  Babl          *babl;
+  int            id         = 0;
+  int            planar     = 0;
+  BablModel     *model      = (BablModel *)babl_model ("Y");
+  BablComponent *component [components];
+  BablSampling  *sampling  [components];
+  const BablType *type      [components];
+  char          *name       = NULL;
+
+  for (i = 0; i<components; i++)
+    {
+      component[i] = model->component[0];
+      type[i] = &btype->type;
+      sampling[i] = (BablSampling *) babl_sampling (1, 1);
+    }
+
+  name = ncomponents_create_name (btype, components);
+  babl = babl_db_exist (db, id, name);
+  if (babl)
+    {
+      /* There is an instance already registered by the required id/name,
+       * returning the preexistent one instead.
+       */
+      babl_free (name);
+      return babl;
+    }
+
+  babl = format_new (name,
+                     id,
+                     planar, components, model,
+                     component, sampling, type);
+
+  babl_format_set_is_format_n (babl);
+
+  babl_db_insert (db, babl);
+  babl_free (name);
+  return babl;
+}
+
+int
+babl_format_is_format_n (const Babl *format)
+{
+  if (format->class_type == BABL_FORMAT)
+    {
+      return format->format.format_n;
+    }
+
+  return 0;
+}
+
+
+static int
+is_format_duplicate (Babl            *babl,
+                     int              planar,
+                     int              components,
+                     BablModel       *model,
+                     BablComponent  **component,
+                     BablSampling   **sampling,
+                     const BablType **type)
+{
+  int i;
+
+  if (babl->format.planar     != planar     ||
+      babl->format.components != components ||
+      babl->format.model      != model)
+    return 0;
+
+  for (i = 0; i < components; i++)
+    {
+      if (babl->format.component[i] != component[i] ||
+          babl->format.sampling[i]  != sampling[i]  ||
+          babl->format.type[i]      != type[i])
+        return 0;
+    }
+  return 1;
+}
+
+const Babl *
+babl_format_new (const void *first_arg,
                  ...)
 {
   va_list        varg;
@@ -192,12 +301,12 @@ babl_format_new (void *first_arg,
   BablModel     *model      = NULL;
   BablComponent *component [BABL_MAX_COMPONENTS];
   BablSampling  *sampling  [BABL_MAX_COMPONENTS];
-  BablType      *type      [BABL_MAX_COMPONENTS];
+  const BablType*type      [BABL_MAX_COMPONENTS];
 
   BablSampling  *current_sampling = (BablSampling *) babl_sampling (1, 1);
   BablType      *current_type     = (BablType *) babl_type_from_id (BABL_DOUBLE);
   char          *name             = NULL;
-  void          *arg              = first_arg;
+  const void    *arg              = first_arg;
 
   va_start (varg, first_arg);
 
@@ -302,12 +411,34 @@ babl_format_new (void *first_arg,
   if (!name)
     name = create_name (model, components, component, type);
 
+  if (!model)
+    {
+      babl_log ("no model specified for format '%s'", name);
+      babl_free(name);
+      return NULL;
+    }
+
+  if (!components)
+    {
+      babl_log ("no components specified for format '%s'", name);
+      babl_free(name);
+      return NULL;
+    }
+
   babl = babl_db_exist (db, id, name);
+  if (id && !babl && babl_db_exist (db, 0, name))
+    babl_fatal ("Trying to reregister BablFormat '%s' with different id!", name);
+
   if (babl)
     {
       /* There is an instance already registered by the required id/name,
-       * returning the preexistent one instead.
+       * returning the preexistent one instead if it doesn't differ.
        */
+      if (!is_format_duplicate (babl, planar, components, model,
+                                component, sampling, type))
+        babl_fatal ("BablFormat '%s' already registered "
+                    "with different content!", name);
+
       babl_free (name);
       return babl;
     }
@@ -317,9 +448,6 @@ babl_format_new (void *first_arg,
                      planar, components, model,
                      component, sampling, type);
 
-  /* Since there is not an already registered instance by the required
-   * id/name, inserting newly created class into database.
-   */
   babl_db_insert (db, babl);
   babl_free (name);
   return babl;
@@ -370,7 +498,7 @@ babl_format_get_n_components (const Babl *format)
   return 0;
 }
 
-Babl *
+const Babl *
 babl_format_get_type (const Babl *format,
                       int         component_index)
 {
@@ -384,9 +512,9 @@ babl_format_get_type (const Babl *format,
   return NULL;
 }
 
-Babl *
-babl_format_with_model_as_type (Babl *model,
-                                Babl *type)
+const Babl *
+babl_format_with_model_as_type (const Babl *model,
+                                const Babl *type)
 {
   BablComponent *component[10];
   int            i;
@@ -433,7 +561,7 @@ test_create (void)
 }
 
 double
-babl_format_loss (Babl *babl)
+babl_format_loss (const Babl *babl)
 {
   double  loss = 0.0;
   double *test;
@@ -442,10 +570,10 @@ babl_format_loss (Babl *babl)
   void   *destination;
   double *transformed;
 
-  Babl   *ref_fmt;
-  Babl   *fmt;
-  Babl   *fish_to;
-  Babl   *fish_from;
+  const Babl *ref_fmt;
+  const Babl *fmt;
+  Babl       *fish_to;
+  Babl       *fish_from;
 
   ref_fmt = babl_format_new (
     babl_model ("RGBA"),
@@ -487,8 +615,42 @@ babl_format_loss (Babl *babl)
   babl_free (transformed);
   babl_free (test);
 
-  babl->format.loss = loss;
+  ((Babl*)babl)->format.loss = loss;
   return loss;
 }
 
+
+void *
+babl_get_user_data (const Babl *babl)
+{
+  switch (babl->instance.class_type)
+    {
+      case BABL_MODEL:
+        return babl->model.data;
+      case BABL_FORMAT:
+        return babl->format.model->data;
+      default:
+        babl_fatal ("babl_get_user_data called on non-model/format");
+    }
+  babl_fatal ("eeeek");
+  return NULL;
+}
+
+void
+babl_set_user_data (const Babl *cbabl, void *data)
+{
+  Babl *babl = (Babl*) cbabl;
+  switch (cbabl->instance.class_type)
+    {
+      case BABL_MODEL:
+        babl->model.data = data;
+        break;
+      case BABL_FORMAT:
+        babl->format.model->data = data;
+        break;
+      default:
+        babl_fatal ("babl_set_user_data called on non-model/format");
+    }
+}
+
 BABL_CLASS_IMPLEMENT (format)
