@@ -323,67 +323,6 @@ conv_rgb8_rgbaF (unsigned char *src, unsigned char *dst, long samples)
   return samples;
 }
 
-static long
-conv_rgbAF_sdl32 (unsigned char *srcc,
-                  unsigned char *dstc,
-                  long           samples)
-{
-  float         *src = (void *) srcc;
-  unsigned char *dst = (void *) dstc;
-  long           n   = samples;
-
-  while (n--)
-    {
-      int   i;
-      float alpha = src[3];
-      for (i = 0; i < 3; i++)
-        {
-          float ca = src[i];
-          int   ret;
-          if (alpha < BABL_ALPHA_THRESHOLD)
-            ret = 0;
-          else
-            ret = table_F_8g[gggl_float_to_index16 (ca / alpha)];
-          if (ret <= 0)
-            dst[i] = 0;
-          else if (ret > 255)
-            dst[i] = 255;
-          else
-            dst[i] = ret;
-        }
-      {
-        int t = dst[0];
-        dst[0] = dst[2];
-        dst[2] = t;
-        dst[3] = 255;
-      }
-      src += 4;
-      dst += 4;
-    }
-  return samples;
-}
-
-
-static long
-conv_rgbaF_sdl32 (unsigned char *srcc,
-                  unsigned char *dstc,
-                  long           samples)
-{
-  float         *src = (void *) srcc;
-  unsigned char *dst = (void *) dstc;
-  long           n   = samples;
-
-  while (n--)
-    {
-      dst[0] = table_F_8g[gggl_float_to_index16 (src[2])];
-      dst[1] = table_F_8g[gggl_float_to_index16 (src[1])];
-      dst[2] = table_F_8g[gggl_float_to_index16 (src[0])];
-      src   += 4;
-      dst   += 4;
-    }
-  return samples;
-}
-
 static long
 conv_rgbAF_rgb8 (unsigned char *srcc,
                  unsigned char *dstc,
@@ -473,9 +412,11 @@ conv_rgbAF_rgbaF (unsigned char *srcc,
   while (n--)
     {
       float alpha = src[3];
-      float recip = 1.0/alpha;
+      float recip;
       if (alpha < BABL_ALPHA_THRESHOLD)
         recip = 0.0;
+      else
+        recip = 1.0/alpha;
       dst[0] = src[0] * recip;
       dst[1] = src[1] * recip;
       dst[2] = src[2] * recip;
@@ -518,8 +459,6 @@ conv_rgbAF_lrgba8 (unsigned char *srcc,
   return samples;
 }
 
-
-
 #define conv_rgb8_rgbAF    conv_rgb8_rgbaF
 
 int init (void);
@@ -527,7 +466,7 @@ int init (void);
 int
 init (void)
 {
-  Babl *rgbaF = babl_format_new (
+  const Babl *rgbaF = babl_format_new (
     babl_model ("RGBA"),
     babl_type ("float"),
     babl_component ("R"),
@@ -535,7 +474,7 @@ init (void)
     babl_component ("B"),
     babl_component ("A"),
     NULL);
-  Babl *rgbAF = babl_format_new (
+  const Babl *rgbAF = babl_format_new (
     babl_model ("RaGaBaA"),
     babl_type ("float"),
     babl_component ("Ra"),
@@ -544,7 +483,7 @@ init (void)
     babl_component ("A"),
     NULL);
 
-  Babl *lrgba8 = babl_format_new (
+  const Babl *lrgba8 = babl_format_new (
     babl_model ("RGBA"),
     babl_type ("u8"),
     babl_component ("R"),
@@ -553,7 +492,7 @@ init (void)
     babl_component ("A"),
     NULL);
 
-  Babl *rgba8 = babl_format_new (
+  const Babl *rgba8 = babl_format_new (
     babl_model ("R'G'B'A"),
     babl_type ("u8"),
     babl_component ("R'"),
@@ -561,7 +500,7 @@ init (void)
     babl_component ("B'"),
     babl_component ("A"),
     NULL);
-  Babl *bgrA8 = babl_format_new (
+  const Babl *bgrA8 = babl_format_new (
     "name", "B'aG'aR'aA u8",
     babl_model ("R'aG'aB'aA"),
     babl_type ("u8"),
@@ -570,22 +509,13 @@ init (void)
     babl_component ("R'a"),
     babl_component ("A"),
     NULL);
-  Babl *rgb8 = babl_format_new (
+  const Babl *rgb8 = babl_format_new (
     babl_model ("R'G'B'"),
     babl_type ("u8"),
     babl_component ("R'"),
     babl_component ("G'"),
     babl_component ("B'"),
     NULL);
-  Babl *sdl32 = babl_format_new (
-    "name", "B'aG'aR'aPAD u8",
-    babl_model ("R'G'B'"),
-    babl_type ("u8"),
-    babl_component ("B'"),
-    babl_component ("G'"),
-    babl_component ("R'"),
-    babl_component ("PAD"),
-    NULL);
 
   table_init ();
 
@@ -598,10 +528,8 @@ init (void)
   o (rgb8, rgbaF);
   o (rgb8, rgbAF);
   o (rgba8, rgbaF);
-  o (rgbaF, sdl32);
   o (rgbaF, rgb8);
   o (rgbAF, rgb8);
-  o (rgbAF, sdl32);
   o (bgrA8, rgba8);
 
   return 0;
